#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
新型コロナウイルス感染者数情報について提供している以下のサイトからデータを取得して解析してみる。
https://github.com/CSSEGISandData/COVID-19
=> Googleで「コロナ 感染者数 データ」で検索して表示される情報の元ネタ。

(処理仕様)
・データは、日毎に mm-dd-yyyy.csv 形式で保存されている
・日本国内のデータのみを集計する(Country_RegionがJapanのもの)
・扱うデータは以下のもの
　　　Confirmed    : 新規感染者数
　　　Deaths       : 死者数
　　　Recovered    : 回復者数
　　　Active       : 感染者数
・対象日が指定された場合はその日のサマリ情報を出力し、指定されない場合は2020.3.22から最新のデータまでを処理対象とする
・指定日のデータが存在しない場合は、それより前で一番近い日付のデータを対象とする
・フルカラーLEDをGPIOに接続する
　デフォルトでは、
　　　赤 -> GPIO23
　　　緑 -> GPIO24
　　　青 -> GPIO25
　に接続されているものとする
・LEDの色判定は以下の通り
　　新規陽性者数(Confirmed)の増加人数(前日比)
　　　　+500以上   赤
　　　　0以下      青
　　　　その他     黄
　　ただし、感染者数(Active)が120000(おおよそ人口の0.1%)を上回っている場合はレベルを引き上げる(+1)

'''

import sys
import os
import datetime
import time
import pandas as pd
import RPi.GPIO as GPIO

# データ採取開始日(2020.3.22)
first_date = datetime.datetime(2020,3,22)

# LED
LED_RED    = 3
LED_YELLOW = 2
LED_BLUE   = 1


# GPIO制御の準備
LED_RED_PIN = 23
LED_GREEN_PIN = 24
LED_BLUE_PIN = 25
GPIO.setmode(GPIO.BCM)	# GPIO番号で指定するモード
GPIO.setup(LED_RED_PIN, GPIO.OUT)
GPIO.setup(LED_GREEN_PIN, GPIO.OUT)
GPIO.setup(LED_BLUE_PIN, GPIO.OUT)

# 点灯間隔
INTERVAL = 1

# デバッグオプション
#debug_mode=True
debug_mode=False


# 指定日に一番近い存在するデータの日付を返す
def target_date(date, first_date):
    target_date = date
    while True:
        if os.path.isfile(os.path.join(arg_dir, target_date.strftime('%m-%d-%Y') + '.csv')):
            break;
        target_date = arg_date + datetime.timedelta(days=-1)
        if first_date == target_date:
            break;

    return target_date


# 指定日のデータを読み込み、PandasのDataFrameを返す
def read_csv(dir, date):
    return pd.read_csv(os.path.join(dir, date.strftime('%m-%d-%Y') + '.csv'))


# 前日との差を求める
def diff(pd_prev, pd_cur):
    # 各カラム(Confirmed,Deaths,Recovered,Active)ごとに合計し日本全体の値とし、前日の差を求める。
    return(pd_cur.sum() - pd_prev.sum())

# 前日からの上昇率を求める
def rate(pd_prev, pd_cur):
    # 各カラム(Confirmed,Deaths,Recovered,Active)ごとに合計し日本全体の値とし、前日からの上昇率を求める。
    return(pd_cur.sum() / pd_prev.sum() * 100 - 100)


# 読み込んだデータの前処理
def preprocess(data):
    # 日本のデータのみを抽出
    a = data.query('Country_Region == "Japan"')

    # 必要なカラムのみ取り出す
    b = a[['Province_State','Confirmed','Deaths','Recovered','Active']]

    # 都道府県をインデックスに設定
    c = b.set_index('Province_State')

    # データがないところ(NaN)がある行は削除することにする
    d = c.dropna()
    return(d)



# LEDを点灯する
def LED_ON(level):
    if debug_mode:
        print('LEVEL = %d' % level)

    if level == LED_BLUE:
        print('BLUE')
        GPIO.output(LED_BLUE_PIN, GPIO.HIGH)
    elif level == LED_YELLOW:
        print('YELLOW')
        GPIO.output(LED_GREEN_PIN, GPIO.HIGH)
        GPIO.output(LED_RED_PIN, GPIO.HIGH)
    else:
        print('RED')
        GPIO.output(LED_RED_PIN, GPIO.HIGH)

# LEDを消灯する
def LED_OFF():
    GPIO.output(LED_RED_PIN, GPIO.LOW)
    GPIO.output(LED_GREEN_PIN, GPIO.LOW)
    GPIO.output(LED_BLUE_PIN, GPIO.LOW)



# オプション解析
arg_dir = ''	# ディレクトリ名
arg_date = ''	# 対象日

if len(sys.argv) < 1:
    print('Usage: %s <directory> [対象日(yyyymmdd)]' %  os.path.basename(sys.argv[0]))
    sys.exit(1)

arg_dir = sys.argv[1]
if len(sys.argv) > 2:
    arg_date = datetime.datetime.strptime(sys.argv[2], '%Y%m%d')


if debug_mode:
    print(arg_dir)
    print(arg_date)

if arg_date != '':	# 日付指定
    # ファイル存在確認。なければその前日のファイルを確認する
    cur_date = target_date(arg_date, first_date)
    prev_date = target_date(cur_date + datetime.timedelta(days=-1), first_date)
    target_date = cur_date
else:
    cur_date = first_date + datetime.timedelta(days=1)
    prev_date = first_date
    target_date = datetime.datetime.today()


# CSVファイルを読み込み解析しLEDを点灯させるループ
while True:
    # まずLEDを消す
    LED_OFF()
    
    if cur_date > target_date:
        break;

    # 処理日
    print(str(cur_date) + ': ', end="")
    
    try:
        # ファイルを読み込んで、データ整形する
        cur = preprocess(read_csv(arg_dir, cur_date))
        prev = preprocess(read_csv(arg_dir, prev_date))

        '''
        prev_dateに対応するファイルは必ず存在する状態にしたいので、ここで設定する。
        cur_dateに対応するファイルがない場合は例外が発生してprev_dateの更新が行われない
        '''
        prev_date = cur_date

    except:
        '''
        データが存在しない場合は、前日と同じにしておく。
        できれば、次に存在するデータと線形補完する方がよいかとは思うが、今回はそこまで頑張らない
        '''
        cur = prev = preprocess(read_csv(arg_dir, prev_date))

    finally:
        # 差を算出
        d1 = diff(prev,cur)
        if debug_mode:
            print(d1)
            
        # 上昇率を算出
        d2 = rate(prev,cur)
        if debug_mode:
            print(d2)

        # レベル判定
        val = d1['Confirmed']
        if val >= 500:
            level = LED_RED
        elif val <= 0:
            level = LED_BLUE
        else:
            level = LED_YELLOW

        # 染者数での補正
        if cur.sum()['Active'] > 120000:
            level = level + 1

        # LEDを点灯させる
        LED_ON(level)

        # しばらく点灯
        time.sleep(INTERVAL)

        # 次の日
        cur_date = cur_date + datetime.timedelta(days=1)


# GPIOの後始末
GPIO.cleanup()

